﻿XIncludeFile "OWire_Module_dll.pbi"
XIncludeFile "OWire_Form.pbf"

#Port=2
Global NewList ListDevices.OW_SN()

Procedure.s GetLastErrorString(Err)
  Protected String.s
  
  Select Err
    Case #OW_Err_OK       : String = "Нет ошибок"
    Case #OW_Err_Port     : String = "Нет открытого COM порта"
    Case #OW_Err_NoDevice : String = "Нет ответа от '1-Wire' устройств"
    Case #OW_Err_IO       : String = "Ошибка обмена данными"
    Case #OW_Err_BaudRate : String = "Ошибка изменения скорости порта"
    Case #OW_Err_Search   : String = "Ошибка при поиске '1-Wire' устройств"
    Case #OW_Err_BadParam : String = "Ошибка в аргументе функции"
    Case #OW_Err_NoData   : String = "Нет данных"
  EndSelect
  
  ProcedureReturn String
EndProcedure

Procedure.s wordform(num.i, form_one.s, form_two.s, form_three.s) 
  num % 100 
  Protected num_two.i = num % 10 
  If num > 10 And num < 20 
    ProcedureReturn form_three 
  EndIf 
  If num_two > 1 And num_two < 5 
    ProcedureReturn form_two 
  EndIf 
  If num_two = 1
    ProcedureReturn form_one 
  EndIf 
  ProcedureReturn form_three 
EndProcedure

Procedure SelectPort()
  
  Port.s=GetGadgetText(#Combo_0)
  If OW_OpenPort(Port, #Port)
    StatusBarText(0, 0, "Порт "+Port+" открыт")
  Else
    StatusBarText(0, 0, "Не удалось открыть порт "+Port)
  EndIf
  
EndProcedure

Procedure SetMode()
  If GetGadgetState(#Checkbox_0)
    OW_SetMode(OW_GetMode() |  #OW_Mode_Invert)
  Else
    OW_SetMode(OW_GetMode() & ~#OW_Mode_Invert)
  EndIf
EndProcedure

Procedure.s SN_Text(*x.OW_SN)
  Protected String.s, i
  
  String = RSet(Hex(*x\Type), 2, "0")+"-"
  
  For i=5 To 0 Step -1
    String + RSet(Hex(*x\Number[i]), 2, "0")
  Next i
  
  String + "-"+RSet(Hex(*x\CRC), 2, "0")
  
  ProcedureReturn String
EndProcedure

Procedure TableUpdate()
  Protected String.s
  
  ClearGadgetItems(#ListIcon_0)
  
  ForEach ListDevices()
    
    Select ListDevices()\Type
      Case $10
        String="DS1820 / DS18S20"
      Case $28
        String="DS18B20"
      Default
        String="Неизвестное устройство"
    EndSelect
    
    String + Chr(10)+SN_Text(@ListDevices())
    If OW_GetCRC(@ListDevices(), 7) <> ListDevices()\CRC
      String+"  ОШИБКА CRC"
    EndIf
    
    AddGadgetItem(#ListIcon_0, -1, String)
    
  Next
  
EndProcedure

Procedure SearchSensors()
  Protected Sensor.OW_SN, Err
  
  ClearList(ListDevices())
  StatusBarText(0, 0, "Поиск '1-Wire' устройств")
  
  OW_SearchROM_Reset()
  Count = 0
  While OW_SearchROM_Enum(@Sensor, #Port)=#True
    Count+1
    If AddElement(ListDevices())
      CopyMemory(@Sensor, @ListDevices(), SizeOf(OW_SN))
    EndIf
  Wend
  
  Err=OW_GetLastError()
  If Err=#OW_Err_OK Or Err=#OW_Err_NoData
    StatusBarText(0, 0, "Найдено "+Count+" '1-Wire' "+
                        wordform(Count, "устройство", " устройства", " устройств"))
  Else
    StatusBarText(0, 0, GetLastErrorString(OW_GetLastError()))
  EndIf
  TableUpdate()
    
EndProcedure

Procedure SetTable(Gadget, Pos, Thermo.s, Status.s)
  SetGadgetItemText(Gadget, Pos, Thermo, 2)
  SetGadgetItemText(Gadget, Pos, Status, 3)
EndProcedure

Procedure GetThermo()
  Protected String.s, i
  Protected RAM.OW_RAM
  Protected Thermo.f, t.w
  
  If ListSize(ListDevices())=0
    StatusBarText(0, 0, "Отсутсвуют '1-Wire' устройства")
    ProcedureReturn
  EndIf
  
  If OW_SkipROM(#Port)
    If OW_ConvertT(#Port)
      
      Delay(800)
      
      ForEach ListDevices()
        If ListDevices()\Type = $10 Or ; DS1820 или DS18S20
           ListDevices()\Type = $28    ; DS18B20
          
          If OW_SelectROM(@ListDevices(), #Port)
            If OW_ReadRAM(@RAM, #Port)
              
              If OW_GetCRC(@RAM, 8)=RAM\CRC
                
                t=(RAM\H_Termo<<8)|RAM\L_Termo
                If ListDevices()\Type = $28 ; DS18B20
                  Thermo=t/16
                Else                        ; DS1820 или DS18S20
                  Thermo=t/2
                EndIf
                
                SetTable(#ListIcon_0, i, StrF(Thermo, 1)+" °C", "ОК")
                
              Else
                SetTable(#ListIcon_0, i, " ", "Ошибка CRC")
              EndIf
              
            Else
              SetGadgetItemText(#ListIcon_0, i, GetLastErrorString(OW_GetLastError()), 3)
            EndIf
          Else
            SetGadgetItemText(#ListIcon_0, i, GetLastErrorString(OW_GetLastError()), 3)
          EndIf
          
        Else
          SetTable(#ListIcon_0, i, " ", "Устройство не поддерживается")
        EndIf
        
        i+1
        
      Next
      
      String="Даные обновлены в "+FormatDate("%hh:%ii:%ss  %dd/%mm/%yyyy", Date())
      
    Else
      String=GetLastErrorString(OW_GetLastError())
    EndIf
  Else
    String=GetLastErrorString(OW_GetLastError())
  EndIf
  
  StatusBarText(0, 0, String)
EndProcedure

Procedure Init()
  
  SmartWindowRefresh(#Window_0, #True)
  WindowBounds(#Window_0, 200, 130, 10000, 10000)
  BindEvent(#PB_Event_SizeWindow, @ResizeGadgetsWindow_0(), #Window_0)
  BindGadgetEvent(#Combo_0, @SelectPort(), #PB_EventType_Change)
  BindGadgetEvent(#Button_0, @SearchSensors())
  BindGadgetEvent(#Button_1, @GetThermo())
  BindGadgetEvent(#Checkbox_0, @SetMode())
  
  For i=0 To 10
    AddGadgetItem(#Combo_0, -1, "/dev/ttyS"+i)
  Next i
  
EndProcedure

CompilerIf #PB_Compiler_OS = #PB_OS_Windows
  
  CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
    #Lib_Name = "OWire_Module.dll"
  CompilerElse
    #Lib_Name = "OWire_Module_x64.dll"
  CompilerEndIf
  
CompilerElseIf #PB_Compiler_OS = #PB_OS_Linux
  
  CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
    #Lib_Name = "OWire_Module.so"
  CompilerElse
    #Lib_Name = "OWire_Module_x64.so"
  CompilerEndIf
  
CompilerEndIf

If OW_Init(GetPathPart(ProgramFilename())+#Lib_Name)=#False
  MessageRequester("", "Ошибка при загрузке "+#Lib_Name)
  End
EndIf

OpenWindow_0()
Init()

Repeat
  Event = WaitWindowEvent()
Until Event = #PB_Event_CloseWindow

OW_End()
; IDE Options = PureBasic 5.22 LTS (Linux - x64)
; CursorPosition = 213
; FirstLine = 206
; Folding = ---
; EnableUnicode
; EnableXP
; UseIcon = Res\thermometer.ico
; Executable = ../OWire
; CompileSourceDirectory
