﻿XIncludeFile "Owire_Module_dll.pbi"

CompilerIf #PB_Compiler_OS = #PB_OS_Windows
  XIncludeFile "DS18B20_Form_Win.pbf"
  #PortName = "COM"
CompilerElse
  XIncludeFile "DS18B20_Form_Lin.pbf"
  #PortName = "/dev/ttyS"
CompilerEndIf

Procedure.s GetLastErrorString(Err)
  Protected String.s
  
  Select Err
    Case #OW_Err_OK       : String = "Нет ошибок"
    Case #OW_Err_Port     : String = "Нет открытого COM порта"
    Case #OW_Err_NoDevice : String = "Нет ответа от '1-Wire' устройств"
    Case #OW_Err_IO       : String = "Ошибка обмена данными"
    Case #OW_Err_BaudRate : String = "Ошибка изменения скорости порта"
    Case #OW_Err_Search   : String = "Ошибка при поиске '1-Wire' устройств"
    Case #OW_Err_BadParam : String = "Ошибка в аргументе функции"
    Case #OW_Err_NoData   : String = "Нет данных"
  EndSelect
  
  ProcedureReturn String
EndProcedure

Procedure Thread(*x)
  Shared OpenPort_Flag
  Protected Info.OW_RAM, x.w
  
  Repeat
    
    If OpenPort_Flag=#True 
      
      OW_SearchROM_Reset()
      Count=0
      While Count<=1 And OW_SearchROM_Enum(@Sensor.OW_SN)=#True
        Count+1
      Wend
      
      If Count>1
        MessageRequester("", "Программа не поддерживает работу с более чем одним"+#LF$+
                             "1-Wire устройством, подключенным к сети.")
      EndIf
      OpenPort_Flag=#False
    EndIf 
    
    If OW_SkipROM() And OW_ConvertT()
      
      Delay(800)
      
      If OW_SkipROM() And OW_ReadRAM(@Info)
        
        If OW_GetCRC(@Info, 8)=Info\CRC
          SetGadgetText(#Text_Status, "Порт открыт")
          x=(Info\H_Termo<<8)|Info\L_Termo
          SetGadgetText(#String_Thermo, StrF(x/16, 1)+" °C")
        Else
          SetGadgetText(#Text_Status, "Ошибка CRC")
          SetGadgetText(#String_Thermo, "")
        EndIf
        
      Else
        Goto m1
      EndIf
      
    Else
      m1:
      x=OW_GetLastError()
      If x<>#OW_Err_OK
        SetGadgetText(#Text_Status, GetLastErrorString(x))
        SetGadgetText(#String_Thermo, "")
      EndIf
    EndIf
    
    Delay(200)
  ForEver
EndProcedure

Procedure ReadSetting()
  OpenPreferences("DS18B20_Thermo.ini")
  Port.s=ReadPreferenceString("Port", "")
  ClosePreferences()
  If Port<>""
    SetGadgetText(#Combo_Ports, Port)
  EndIf
EndProcedure

Procedure WriteSetting()
  CreatePreferences("DS18B20_Thermo.ini")
  WritePreferenceString("Port", GetGadgetText(#Combo_Ports))
  ClosePreferences()
EndProcedure

Procedure EnumPorts()
  ClearGadgetItems(#Combo_Ports)
  For i=0 To 10
    If OpenSerialPort(2, #PortName+i, 9600, #PB_SerialPort_NoParity,
                      8, 1, #PB_SerialPort_NoHandshake, 255, 255)
      AddGadgetItem(#Combo_Ports, -1, #PortName+i)
      CloseSerialPort(2)
    EndIf
  Next
EndProcedure

Procedure SelectPort()
  Shared OpenPort_Flag
  
  Port.s = GetGadgetText(#Combo_Ports)
  
  OpenPort_Flag = #False
  
  If Port<>""
    If OW_OpenPort(Port)
      OpenPort_Flag = #True
      SetGadgetText(#Text_Status, "Порт открыт")
      
    Else
      SetGadgetText(#Text_Status, GetLastErrorString(OW_GetLastError()))
      SetGadgetText(#String_Thermo, "")
    EndIf
  EndIf
  
EndProcedure

SetCurrentDirectory(GetPathPart(ProgramFilename()))

OpenWinMain()
EnumPorts()
ReadSetting()
SelectPort()

ThreadNum = CreateThread(@Thread(), 0)
BindGadgetEvent(#Combo_Ports, @SelectPort(), #PB_EventType_Change)
HideWindow(#WinMain, #False)

Repeat
  Event = WaitWindowEvent()
Until Event = #PB_Event_CloseWindow
KillThread(ThreadNum)
WriteSetting()

OW_ClosePort()
; IDE Options = PureBasic 5.21 LTS (Windows - x64)
; CursorPosition = 40
; FirstLine = 17
; Folding = --
; EnableUnicode
; EnableThread
; EnableXP
; UseIcon = Res\MainIcon.ico
; Executable = ..\DS18B20_Thermo.exe