﻿
; Коды ошибок (возвращает функция OW_GetLastErr()).
Enumeration
  #OW_Err_OK        ; Ошибок нет.
  #OW_Err_Port      ; Нет открытого порта (для OW_OpenPort() - ошибка открытия порта).
  #OW_Err_NoDevice  ; Не найдены 1-Wire устройства.
  #OW_Err_IO        ; Ошибка при обмене информацией.
  #OW_Err_BaudRate  ; Ошибка при изменении скорости порта.
  #OW_Err_Search    ; Ошибка происходит если во время поиска ни одно 1-Wire устройство не ответило.
  #OW_Err_BadParam  ; Некорректный параметр.
  #OW_Err_NoData    ; Нет данных.
EndEnumeration

; Режим работы модуля.
#OW_Mode_CPU_MinUsage = 1  ; Уменьшить загрузку процессора.
#OW_Mode_Invert       = 2  ; Инвертирование сигналов DTR и RTS.
#OW_Mode_NoControlPin = 4  ; Не изменять состояние управляющих выходов DTR и RTS.

; Управление выходами порта (функция SP_SetControlPin()).
#SP_Pin_DTR = 1
#SP_Pin_RTS = 2
#SP_Pin_TXD = 4
; Чтение состояния входов порта (функция SP_GetControlPin()).
#SP_Pin_RI  = 0
#SP_Pin_DCD = 1
#SP_Pin_DSR = 2
#SP_Pin_CTS = 3

Structure OW_RAM ; ОЗУ датчика DS18B20.
  L_Termo.a       ; Младший байт температуры
  H_Termo.a       ; Старший байт температуры
  Max_Termo.b     ; Верхний предел температуры
  Min_Termo.b     ; Нижний предел температуры
  Config.a        ; Байт конфигурации
  x.a[3]          ; Резерв, 3 байта (в датчике не реализовано)
  CRC.a           ; Контрольная сумма
EndStructure

Structure OW_WriteRAM
  Max_Termo.b     ; Верхний предел температуры.
  Min_Termo.b     ; Нижний предел температуры.
  Config.a        ; Байт конфигурации.
EndStructure

Structure OW_SN  ; Идентификатор датчика.
  Type.a          ; Тип устройства (код семейства).
  Number.a[6]     ; Серийный номер.
  CRC.a           ; Контрольная сумма.
EndStructure

CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
  Import "OWire_Module.lib"
CompilerElseIf #PB_Compiler_Processor = #PB_Processor_x64
  Import "OWire_Module_x64.lib"
CompilerEndIf

  OW_GetMode()
  OW_SetMode(Mode)     ; Режим. См. константы #OW_Mode_xxxx .
  OW_GetLastError()    ; Код ошибки (см. константы #OW_Err_xxxx и #SP_Err_xxxx).
  OW_ClosePort(Port=0) ; Закрывает последовательный порт.
  OW_OpenPort(ComPort.p-ascii, Port=0) ; Открывает последовательный порт.
  OW_Reset(Port=0)    ; Сброс 1-Wire устройства.
  OW_ReadByte.a(Port=0) ; Чтение байта.
  OW_WriteByte.a(Byte.a, Port=0) ; Запись байта.
  OW_GetCRC.a(*Buff, Count)    ; Вычисление контрольной суммы.
  
  OW_ReadMultiByte(*Buff, Count, Port=0) ; Чтение массива байт.
  OW_WriteMultiByte(*Buff, Count, Port=0) ; Запись массива байт.
    
  ; 1-Wire устройство передает 8-битный код семейства (10h для DS18S20 и 28h для DS18B20),
  ; затем 48-битный серийный номер, а затем 8-битную CRC для проверки правильности принятой информации.
  OW_ReadROM(*Info.OW_SN, Port=0)
  
  ; Эта функция позволяет адресовать на шине требуемое 1-Wire устройство.
  ; Необходимо передать 64-битный код 1-Wire устройства и только то,
  ; что имеет такой код, будет «откликаться» до следующего импульса сброса.
  OW_SelectROM(*Info.OW_SN, Port=0)
  
  OW_SkipROM(Port=0) ; Пропуск сравнения серийного номера.
  
  OW_SearchROM_Reset() ; Поиск 1-Wire устройств.
  OW_SearchROM_Enum(*Sensor.OW_SN, Port=0) ; Перечисление устройств. Вернет #True если еще есть устройства.
  
  ; Поиск температурных датчиков у которых результат последнего измерения температуры
  ; выходит за предустановленные пределы TH и TL.
  OW_SearchAlarm_Reset()
  OW_SearchAlarm_Enum(*Sensor.OW_SN, Port=0) ; Перечисление устройств. Вернет #True если еще есть устройства.
  
  ; Чтение данных из промежуточного ОЗУ датчика DS18B20.
  OW_ReadRAM(*Info.OW_RAM, Port=0)
  
  ; Запись данных в промежуточное ОЗУ датчика DS18B20.
  OW_WriteRAM(*Info.OW_WriteRAM, Port=0)
  
  ; Копирование данных из промежуточного ОЗУ в энергонезависимую память датчика.
  ; Это требует около 10мс.
  OW_CopyRAM(Port=0)
  
  ; Запуск процесса преобразования температуры.
  OW_ConvertT(Port=0)
  
  ; Эта функция действует обратным образом по отношению к процедуре OW_CopyRAM(),
  ; т.е. она позволяет считать байты TH и TL из энергонезависимой памяти в промежуточное ОЗУ.
  OW_Recall(Port=0)
  
  ; Проверка, использует ли 1-Wire устройство паразитное питание.
  ; 1 - не использует или 0 если использует.
  OW_ReadPowerSupply(Port=0)
  
  SP_PortID(Port=0) ; Функция возвращает дескриптор порта.
  SP_CountInBytes(Port=0) ; Функция возвращает количество байт в приемном буфере порта.
  SP_ReadByte.a(Port=0) ; Чтение байта из порта.
  SP_WriteByte(Byte.a, Port=0) ; Запись байта в порт.
  SP_GetControlPin(Pin, Port=0) ; Чтение состояния управляющих входов.
  SP_SetControlPin(Pin, Value.a, Port=0) ; Изменение состояния управляющих выходов.
EndImport


; Procedure OW_SearchROM(List SN.OW_SN(), Port)
;   Protected Count=0, Sensor.OW_SN
;   
;   If ListSize(SN())
;     ClearList(SN())
;   EndIf
;   
;   OW_SearchROM_Reset()
;   
;   While OW_SearchROM_Enum(@Sensor, Port)=#True
;     Count+1
;     If AddElement(SN())
;       CopyMemory(@Sensor, @SN(), SizeOf(OW_SN))
;     EndIf
;   Wend
;   
;   ProcedureReturn Count
; EndProcedure
; 
; Procedure OW_SearchAlarm(List SN.OW_SN(), Port)
;   Protected Count=0, Sensor.OW_SN
;   
;   If ListSize(SN())
;     ClearList(SN())
;   EndIf
;   
;   OW_SearchAlarm_Reset()
;   
;   While OW_SearchAlarm_Enum(@Sensor, Port)=#True
;     Count+1
;     If AddElement(SN())
;       CopyMemory(@Sensor, @SN(), SizeOf(OW_SN))
;     EndIf
;   Wend
;   
;   ProcedureReturn Count
; EndProcedure
; IDE Options = PureBasic 5.21 LTS (Windows - x86)
; CursorPosition = 154
; FirstLine = 113
; EnableXP